@@ -19,7 +19,7 @@ class AttributeChild(TemplateElement):
     def __init__(self,
             docgetter: util.DocGetter,
             ob: Attribute,
-            extras: "Flattenable",
+            extras: List[Tag],
             loader: ITemplateLoader
             ):
         super().__init__(loader)
@@ -62,21 +62,13 @@ class AttributeChild(TemplateElement):
             return ()
 
     @renderer
-    def functionExtras(self, request: object, tag: Tag) -> "Flattenable":
+    def objectExtras(self, request: object, tag: Tag) -> List[Tag]:
         return self._functionExtras
 
     @renderer
     def functionBody(self, request: object, tag: Tag) -> "Flattenable":
         return self.docgetter.get(self.ob)
 
-    @renderer
-    def functionDeprecated(self, request: object, tag: Tag) -> "Flattenable":
-        msg = self.ob._deprecated_info
-        if msg is None:
-            return ()
-        else:
-            return tags.div(msg, role="alert", class_="deprecationNotice alert alert-warning")
-
     @renderer
     def constantValue(self, request: object, tag: Tag) -> "Flattenable":
         if self.ob.kind is not DocumentableKind.CONSTANT or self.ob.value is None:
