@@ -158,7 +158,9 @@ def test_missing_variable() -> None:
 
 @pytest.mark.parametrize(
     'className',
-    ['NewClassThatMultiplyInherits', 'OldClassThatMultiplyInherits'],
+    ['NewClassThatMultiplyInherits', 
+     'OldClassThatMultiplyInherits',
+     'Diamond'],
 )
 def test_multipleInheritanceNewClass(className: str) -> None:
     """
@@ -173,11 +175,45 @@ def test_multipleInheritanceNewClass(className: str) -> None:
         if cls.name == className
     )
 
+    assert isinstance(cls, model.Class)
     html = getHTMLOf(cls)
 
     assert "methodA" in html
     assert "methodB" in html
 
+    getob = system.allobjects.get
+
+    if className == 'Diamond':
+        assert util.class_members(cls) == [
+            (
+                (getob('multipleinheritance.mod.Diamond'),),
+                [getob('multipleinheritance.mod.Diamond.newMethod')]
+            ),
+            (
+                (getob('multipleinheritance.mod.OldClassThatMultiplyInherits'),
+                 getob('multipleinheritance.mod.Diamond')),
+                [getob('multipleinheritance.mod.OldClassThatMultiplyInherits.methodC')]
+            ),
+            (
+                (getob('multipleinheritance.mod.OldBaseClassA'),
+                getob('multipleinheritance.mod.OldClassThatMultiplyInherits'),
+                getob('multipleinheritance.mod.Diamond')),
+                [getob('multipleinheritance.mod.OldBaseClassA.methodA')]),
+                ((getob('multipleinheritance.mod.OldBaseClassB'),
+                getob('multipleinheritance.mod.OldBaseClassA'),
+                getob('multipleinheritance.mod.OldClassThatMultiplyInherits'),
+                getob('multipleinheritance.mod.Diamond')),
+                [getob('multipleinheritance.mod.OldBaseClassB.methodB')]),
+                ((getob('multipleinheritance.mod.CommonBase'),
+                getob('multipleinheritance.mod.NewBaseClassB'),
+                getob('multipleinheritance.mod.NewBaseClassA'),
+                getob('multipleinheritance.mod.NewClassThatMultiplyInherits'),
+                getob('multipleinheritance.mod.OldBaseClassB'),
+                getob('multipleinheritance.mod.OldBaseClassA'),
+                getob('multipleinheritance.mod.OldClassThatMultiplyInherits'),
+                getob('multipleinheritance.mod.Diamond')),
+                [getob('multipleinheritance.mod.CommonBase.fullName')]) ]
+
 def test_html_template_version() -> None:
     lookup = TemplateLookup(template_dir)
     for template in lookup._templates.values():
@@ -523,8 +559,8 @@ def test_compact_module_summary() -> None:
     assert ul.tagName == 'ul'       # type: ignore
     assert len(ul.children) == 50   # type: ignore
 
-    # the 51th module triggers the compact summary
-    fromText('', parent_name='top', modname='_yet_another_sub', system=system)
+    # the 51th module triggers the compact summary, no matter if it's a package or module
+    fromText('', parent_name='top', modname='_yet_another_sub', system=system, is_package=True)
 
     ul = moduleSummary(top, '').children[-1]
     assert ul.tagName == 'ul'       # type: ignore
@@ -533,8 +569,8 @@ def test_compact_module_summary() -> None:
     # test that the last module is private
     assert 'private' in ul.children[0].children[-1].attributes['class'] # type: ignore
 
-    # for the compact summary no submodule may have further submodules
-    fromText('', parent_name='top.sub33', modname='subsubmodule', system=system)
+    # for the compact summary no submodule (packages) may have further submodules
+    fromText('', parent_name='top._yet_another_sub', modname='subsubmodule', system=system)
 
     ul = moduleSummary(top, '').children[-1]
     assert ul.tagName == 'ul'       # type: ignore
@@ -562,9 +598,10 @@ def test_index_contains_infos(tmp_path: Path) -> None:
               '<a href="https://github.com/twisted/pydoctor/">pydoctor</a>',)
 
     system = model.System()
-    system.addPackage(testpackages / "allgames")
-    system.addPackage(testpackages / "basic")
-    system.process()
+    builder = system.systemBuilder(system)
+    builder.addModule(testpackages / "allgames")
+    builder.addModule(testpackages / "basic")
+    builder.buildModules()
     w = writer.TemplateWriter(tmp_path, TemplateLookup(template_dir))
     w.writeSummaryPages(system)
 
